##############################################################################
#
# SubPlone - Simple, nested sites to go
# Copyright (C) 2005 Sidnei da Silva and contributors
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
##############################################################################
"""
$Id: Install.py 8663 2005-05-23 16:32:15Z dreamcatcher $
"""

from cStringIO import StringIO
from Products.CMFCore.DirectoryView import addDirectoryViews
from Products.CMFCore.utils import getToolByName, manage_addTool
from Products.SubPlone.config import *
from Products.SubPlone.permissions import ModifyPortalContent
from Products.Archetypes.public import listTypes
from Products.Archetypes.Extensions.utils import installTypes

def install_types(self, out):
    types = listTypes(PROJECTNAME)
    installTypes(self, out, types, PROJECTNAME)

def install_skin(self, out):
    skinsTool = getToolByName(self, 'portal_skins')

    layer_name = LAYER_NAME
    layer_location = '%s/skins' % PROJECTNAME

    # add in the directory view pointing to our skin
    if layer_name not in skinsTool.objectIds():
        addDirectoryViews(skinsTool, 'skins', GLOBALS)
        print >> out, 'Added "%s" directory view to portal_skins' % layer_name

    # add in the layer to all our skins
    skins = skinsTool.getSkinSelections()
    for skin in skins:
        path = skinsTool.getSkinPath(skin)
        path = [p.strip() for p in path.split(',')]
        if layer_name not in path:
            position = 'custom' in path and (path.index('custom') + 1) or 0
            path.insert(position, layer_name)
            path = ', '.join(path)
            skinsTool.addSkinSelection(skin, path)
            print >> out, 'Added "%s" to "%s" skin' % (layer_name, skin)
        else:
            print >> out, 'Skin "%s" already contained %s' % (skin, layer_name)

def install_add_ref_action(self, out):
    at = getToolByName(self, 'portal_actions')
    if 'add_reference' not in [action.getId() for action in at.listActions()]:
        at.addAction('add_reference',
                     'Add a reference to a subsite',
                     'string:${object_url}/sub_add_reference_form',
                     "python:object.getPortalTypeName() not in ('SubSite',)",
                     ModifyPortalContent,
                     'document_actions')
        out.write('Added action.\n')
    else:
        out.write('Action already installed.\n')
    ai = getToolByName(self, 'portal_actionicons')
    if ai.queryActionIcon('plone', 'add_reference', None) is None:
        ai.addActionIcon('plone', 'add_reference',
                         'folder_icon.gif',
                         'Add a reference to a subsite')
        out.write('Added action icon.\n')
    else:
        out.write('Action icon already installed.\n')

def install(self, out=None):
    if out is None:
        out = StringIO()
    install_skin(self, out)
    install_types(self, out)
    install_add_ref_action(self, out)
    return out.getvalue()
