##############################################################################
#
# SubPlone - Simple, nested sites to go
# Copyright (C) 2005 Sidnei da Silva and contributors
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
##############################################################################
"""
$Id: site.py 19257 2006-02-13 21:36:47Z justizin $
"""

from AccessControl import ClassSecurityInfo
from ComputedAttribute import ComputedAttribute
from Products.CMFCore.utils import getToolByName
from Products.SubPlone.config import *
from Products.SubPlone.permissions import View, AddReference
from Products.SubPlone.mixin.proxy import ProxyContainer
from Products.SubPlone.mixin.skin import Skin
from Products.SubPlone.utils import Pipe, StopProcessing
from Products.SubPlone.utils import ListAggregator, LastAggregator
from Products.SubPlone.utils import FirstAggregator, SumAggregator
from Products.SubPlone.utils import OrAggregator
from Products.SubPlone.registry import RefRegistry
from Products.ATContentTypes.content.folder import ATBTreeFolder
from Products.Archetypes.public import BaseBTreeFolder, registerType
from Products.Archetypes.public import StringField, SelectionWidget
from Products.Archetypes.public import Schema, StringWidget, DisplayList
from Products.TALESField import TALESString

class Site(BaseBTreeFolder, Skin, ProxyContainer):
    """ Nested Site
    """

    security = ClassSecurityInfo()
    ok = {'proxy': 0,
          'archetype_name':1,
          'portal_type':1,
          'meta_type':1,
          'title':1,
          'getCharset':1}

    _at_rename_after_creation = 1
    # XXX Not quite sure why it fails, I suspect it's the bug Stephan
    # is tracking on the security policy.
    # security.setDefaultAccess(ok)

    archetype_name = portal_type = meta_type = SITE_TYPE
    schema = BaseBTreeFolder.schema + Schema((
        StringField('skin_name',
                    accessor='getSkinName',
                    mutator='setSkinName',
                    required=False,
                    default=None,
                    vocabulary='_get_skins',
                    enforceVocabulary=True,
                    widget=SelectionWidget(label='Select Skin')),
        TALESString('skin_condition',
                    accessor='getSkinCondition',
                    mutator='setSkinCondition',
                    widget=StringWidget(
        label='Skin Condition',
        description=('A TALES expression to be evaluated '
                     'and check if the skin will be used or not'))
                    ),
        ))

    def __init__(self, oid, **kw):
        ProxyContainer.__init__(self)
        BaseBTreeFolder.__init__(self, oid, **kw)

    objectIds = Pipe('objectIds', BaseBTreeFolder,
                     aggregator=ListAggregator(),
                     before=[('ProxyContainer.objectIds',
                              ProxyContainer.objectIds)]
                     )

    objectValues = Pipe('objectValues', BaseBTreeFolder,
                     aggregator=ListAggregator(),
                     before=[('ProxyContainer.objectValues',
                              ProxyContainer.objectValues)]
                     )

    objectItems = Pipe('objectItems', BaseBTreeFolder,
                     aggregator=ListAggregator(),
                     before=[('ProxyContainer.objectItems',
                              ProxyContainer.objectItems)]
                     )

    objectCount = Pipe('objectCount', BaseBTreeFolder,
                     aggregator=SumAggregator(),
                     before=[('ProxyContainer.objectCount',
                              ProxyContainer.objectCount)]
                     )

    has_key = Pipe('has_key', BaseBTreeFolder,
                   aggregator=OrAggregator(),
                   before=[('ProxyContainer.has_key',
                            ProxyContainer.has_key)]
                   )

    # _getOb is tricky because it may be called with a non-marker
    # 'default' argument, so we use a different function in order to
    # handle this and not modify the ProxyContainer._getOb.
    def _proxy_getOb(self, key, default=marker):
        res = ProxyContainer._getOb(self, key, default)
        if res is default:
            return marker
        return res

    _getOb = Pipe('_getOb', BaseBTreeFolder,
                  aggregator=FirstAggregator(),
                  before=[('ProxyContainer._getOb',
                           _proxy_getOb)]
                  )

    _delOb = Pipe('_delOb', BaseBTreeFolder,
                  aggregator=FirstAggregator(),
                  before=[('ProxyContainer._delOb',
                           ProxyContainer._delOb)]
                  )

    _delObject = Pipe('_delObject', BaseBTreeFolder,
                  aggregator=FirstAggregator(),
                  before=[('ProxyContainer._delObject',
                           ProxyContainer._delObject)]
                  )

    manage_afterAdd = Pipe('manage_afterAdd', BaseBTreeFolder,
                           aggregator=LastAggregator(),
                           before=[('Skin.manage_afterAdd',
                                    Skin.manage_afterAdd)])

    manage_beforeDelete = Pipe('manage_beforeDelete', BaseBTreeFolder,
                               aggregator=LastAggregator(),
                               before=[('Skin.manage_beforeDelete',
                                        Skin.manage_beforeDelete)])

    def __getitem__(self, key):
        try:
            return ProxyContainer.__getitem__(self, key)
        except KeyError:
            return BaseBTreeFolder.__getitem__(self, key)

    def __bobo_traverse__(self, REQUEST, name, RESPONSE=None):
        """ Temporary fix for Archetypes's __bobo_traverse__ not
        looking at __getitem__.
        """
        bt = BaseBTreeFolder.__bobo_traverse__
        try:
            return bt(self, REQUEST, name)
        except AttributeError:
            return self[name]

    security.declareProtected(AddReference, 'addProxyReference')
    def addProxyReference(self, ref_type, name, *args, **kw):
        info = RefRegistry[ref_type]
        ref = info.create(*args, **kw)
        self.proxy.add(name, ref)

    def _get_skins(self):
        st = getToolByName(self, 'portal_skins')
        skins = st.getSkinSelections()
        return DisplayList(tuple(zip(skins, skins)))

    security.declareProtected(View, 'condition')
    def condition(self, container, request):
        # Arguments are ignored right now.
        return self.getSkinCondition()

registerType(Site, PROJECTNAME)

class ATSite(ATBTreeFolder, Skin, ProxyContainer):
    """ Nested Site
    """

    security = ClassSecurityInfo()
    ok = {'proxy': 0,
          'archetype_name':1,
          'portal_type':1,
          'meta_type':1,
          'title':1,
          'getCharset':1}

    _at_rename_after_creation = 1
    # XXX Not quite sure why it fails, I suspect it's the bug Stephan
    # is tracking on the security policy.
    # security.setDefaultAccess(ok)

    archetype_name = portal_type = meta_type = SITE_TYPE
    schema = ATBTreeFolder.schema + Schema((
        StringField('skin_name',
                    accessor='getSkinName',
                    mutator='setSkinName',
                    required=False,
                    default=None,
                    vocabulary='_get_skins',
                    enforceVocabulary=True,
                    widget=SelectionWidget(label='Select Skin')),
        TALESString('skin_condition',
                    accessor='getSkinCondition',
                    mutator='setSkinCondition',
                    widget=StringWidget(
        label='Skin Condition',
        description=('A TALES expression to be evaluated '
                     'and check if the skin will be used or not'))
                    ),
        ))

    def __init__(self, oid, **kw):
        ProxyContainer.__init__(self)
        BaseBTreeFolder.__init__(self, oid, **kw)

    objectIds = Pipe('objectIds', ATBTreeFolder,
                     aggregator=ListAggregator(),
                     before=[('ProxyContainer.objectIds',
                              ProxyContainer.objectIds)]
                     )

    objectValues = Pipe('objectValues', ATBTreeFolder,
                     aggregator=ListAggregator(),
                     before=[('ProxyContainer.objectValues',
                              ProxyContainer.objectValues)]
                     )

    objectItems = Pipe('objectItems', ATBTreeFolder,
                     aggregator=ListAggregator(),
                     before=[('ProxyContainer.objectItems',
                              ProxyContainer.objectItems)]
                     )

    objectCount = Pipe('objectCount', ATBTreeFolder,
                     aggregator=SumAggregator(),
                     before=[('ProxyContainer.objectCount',
                              ProxyContainer.objectCount)]
                     )

    has_key = Pipe('has_key', ATBTreeFolder,
                   aggregator=OrAggregator(),
                   before=[('ProxyContainer.has_key',
                            ProxyContainer.has_key)]
                   )

    # _getOb is tricky because it may be called with a non-marker
    # 'default' argument, so we use a different function in order to
    # handle this and not modify the ProxyContainer._getOb.
    def _proxy_getOb(self, key, default=marker):
        res = ProxyContainer._getOb(self, key, default)
        if res is default:
            return marker
        return res

    _getOb = Pipe('_getOb', ATBTreeFolder,
                  aggregator=FirstAggregator(),
                  before=[('ProxyContainer._getOb',
                           _proxy_getOb)]
                  )

    _delOb = Pipe('_delOb', ATBTreeFolder,
                  aggregator=FirstAggregator(),
                  before=[('ProxyContainer._delOb',
                           ProxyContainer._delOb)]
                  )

    _delObject = Pipe('_delObject', ATBTreeFolder,
                  aggregator=FirstAggregator(),
                  before=[('ProxyContainer._delObject',
                           ProxyContainer._delObject)]
                  )

    manage_afterAdd = Pipe('manage_afterAdd', ATBTreeFolder,
                           aggregator=LastAggregator(),
                           before=[('Skin.manage_afterAdd',
                                    Skin.manage_afterAdd)])

    manage_beforeDelete = Pipe('manage_beforeDelete', ATBTreeFolder,
                               aggregator=LastAggregator(),
                               before=[('Skin.manage_beforeDelete',
                                        Skin.manage_beforeDelete)])

    def __getitem__(self, key):
        try:
            return ProxyContainer.__getitem__(self, key)
        except KeyError:
            return ATBTreeFolder.__getitem__(self, key)

    def __bobo_traverse__(self, REQUEST, name, RESPONSE=None):
        """ Temporary fix for Archetypes's __bobo_traverse__ not
        looking at __getitem__.
        """
        bt = ATBTreeFolder.__bobo_traverse__
        try:
            return bt(self, REQUEST, name)
        except AttributeError:
            return self[name]

    security.declareProtected(AddReference, 'addProxyReference')
    def addProxyReference(self, ref_type, name, *args, **kw):
        info = RefRegistry[ref_type]
        ref = info.create(*args, **kw)
        self.proxy.add(name, ref)

    def _get_skins(self):
        st = getToolByName(self, 'portal_skins')
        skins = st.getSkinSelections()
        return DisplayList(tuple(zip(skins, skins)))

    security.declareProtected(View, 'condition')
    def condition(self, container, request):
        # Arguments are ignored right now.
        return self.getSkinCondition()

registerType(ATSite, PROJECTNAME)
