##############################################################################
#
# SubPlone - Simple, nested sites to go
# Copyright (C) 2005 Sidnei da Silva and contributors
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
##############################################################################
"""
$Id: blocker.py 12922 2005-10-12 02:29:48Z dreamcatcher $
"""

import os
from zExceptions import Unauthorized
from Globals import Persistent
from ExtensionClass import Base
from Acquisition import Implicit, aq_inner, aq_parent, aq_base
from OFS.Traversable import Traversable
from ZPublisher.BeforeTraverse import registerBeforeTraverse
from ZPublisher.BeforeTraverse import unregisterBeforeTraverse
from ZPublisher.BeforeTraverse import NameCaller
from Products.CMFCore.utils import _checkPermission
from Products.SiteAccess.AccessRule import AccessRule
from Products.SubPlone.config import *
from Products.SubPlone.permissions import ModifyPortalContent

SUPPRESS_ACCESSRULE = os.environ.has_key('SUPPRESS_ACCESSRULE')

class EnvironmentSuppressAccessRule(NameCaller):
    """A AccessRule that can be disabled only by environment but not
    by URL mangling.
    """
    meta_type = 'Environment Suppress Access Rule'

    def __call__(self, container, request):
        if SUPPRESS_ACCESSRULE: return
        NameCaller.__call__(self, container, request)

class AccessChecker(Base):

    def __init__(self, property_name):
        self.property_name = property_name

    def __call__(self, container, request):
        if not getattr(container, self.property_name, True):
            return False
        return True

class DomainAccessChecker(AccessChecker):

    def __call__(self, container, request):
        if not AccessChecker.__call__(self, container, request):
            return False
        server_url = request['SERVER_URL'].lower()
        domains = [d.lower() for d in container.blocker_domains]
        if not domains:
            # Empty means allow any
            return True
        for domain in domains:
            if domain in server_url:
                return True
        log('DomainAccessChecker: %s not in %s' % (server_url, str(domains)),
            level=INFO)
        raise Unauthorized, container.getId()

HOOK_NAME = 'access_checker'
PRIORITY = 3
FLAG = '_access_checker_installed'

class DomainBlocker(Persistent, Traversable, Implicit):
    """ Allow to block access based on domain
    """

    blocker_domains = ()
    access_checking_enabled = True
    access_checker = DomainAccessChecker('access_checking_enabled')
    _hook_factory = EnvironmentSuppressAccessRule

    def __updateTraversalData(self, adding=False):
        if hasattr(aq_base(self), FLAG):
            unregisterBeforeTraverse(self, HOOK_NAME)
            delattr(self, FLAG)

        if adding:
            hook = self._hook_factory(HOOK_NAME)
            registerBeforeTraverse(self, hook, HOOK_NAME, PRIORITY)
            setattr(self, FLAG, True)

    def manage_afterAdd(self, item, container):
        """ Add our traversal hook
        """
        self.__updateTraversalData(True)

    def manage_beforeDelete(self, item, container):
        """ Clean up on delete
        """
        self.__updateTraversalData(False)
