##############################################################################
#
# SubPlone - Simple, nested sites to go
# Copyright (C) 2005 Sidnei da Silva and contributors
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
##############################################################################
"""
$Id: skin.py 9909 2005-07-15 02:12:38Z dreamcatcher $
"""

from Globals import Persistent
from ExtensionClass import Base
from Acquisition import Implicit, aq_inner, aq_parent, aq_base
from OFS.Traversable import Traversable
from ZPublisher.BeforeTraverse import registerBeforeTraverse
from ZPublisher.BeforeTraverse import unregisterBeforeTraverse
from Products.SiteAccess.AccessRule import AccessRule
from Products.SubPlone.config import *

class Condition:

    def __init__(self, property_name):
        self.property_name = property_name

    def __call__(self, container, request):
        return getattr(container, self.property_name, True)

class TrueCondition(Condition):

    def __call__(self, container, request):
        if not Condition.__call__(self, container, request):
            return False
        return True

class MatchingDomainCondition(Condition):

    def __call__(self, container, request):
        if not Condition.__call__(self, container, request):
            return False
        server_url = request['SERVER_URL'].lower()
        domains = [d.lower() for d in container.public_domains]
        if not domains:
            # Empty means allow any
            return False
        for domain in domains:
            if domain in server_url:
                return True
        return False

class ConditionSkinSwitcher(Base):

    def __call__(self, container, request):
        if container.skin_name is None:
            return False
        return not not container.condition(container, request)

class SkinSwitcher(ConditionSkinSwitcher):

    def __call__(self, container, request):
        if ConditionSkinSwitcher.__call__(self, container, request):
            container.changeSkin(container.skin_name)

class FiveSkinSwitcher(ConditionSkinSwitcher):

    def __call__(self, container, request):
        if ConditionSkinSwitcher.__call__(self, container, request):
            request.setPresentationSkin(container.skin_name)

HOOK_NAME = 'skin_switcher'
PRIORITY = 2
FLAG = '_skin_traverser_installed'

class Skin(Persistent, Traversable, Implicit):
    """ Allow to choose current skin based on a condition
    """

    skin_name = None
    skin_switching_enabled = True
    condition = TrueCondition('skin_switching_enabled')
    skin_switcher = SkinSwitcher()

    def __updateTraversalData(self, adding=False):
        if hasattr(aq_base(self), FLAG):
            unregisterBeforeTraverse(self, HOOK_NAME)
            delattr(self, FLAG)

        if adding:
            hook = AccessRule(HOOK_NAME)
            registerBeforeTraverse(self, hook, HOOK_NAME, PRIORITY)
            setattr(self, FLAG, True)

    def manage_afterAdd(self, item, container):
        """ Add our traversal hook
        """
        self.__updateTraversalData(True)

    def manage_beforeDelete(self, item, container):
        """ Clean up on delete
        """
        self.__updateTraversalData(False)
